<?php
namespace Teknisa\OPL\Modules\Consolidacao\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Sup07200LancamentoDePedidosService {
    protected $connection;
    protected $newCode;
    
    public function __construct($entityManager, $environment, $newCode) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->newCode = $newCode;
    }
    
    public function realizaCommit() {
        $this->entityManager->commit();
    }
    
    public function realizaRollback() {
        $this->entityManager->rollBack();
    }
    
    public function beginTransaction() {
        $this->entityManager->beginTransaction();
    }
    
    public function getCentralCompras($codigoFilial = null) {
        if (!$codigoFilial) {
            $codigoFilial = $this->environment->getCdFilial();
        }
        
        $query = "SELECT
                        F.CDFILIAL,
                        F.NMFILIAL,
                        F2.CDFILIAL   AS CDFILCENTCMP,
                        F2.NMFILIAL   AS NMFILCENTCMP,
                        F2.NRORG
                    FROM
                        FILIAL   F,
                        FILIAL   F2
                    WHERE
                        F.CDFILIAL = :CDFILIAL
                        AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "CDFILIAL" => $codigoFilial,
        ));
        return $std->fetch();
    }
    
    public function checkIgualdadeItemPedidoOL($row) {
        $param = array();
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        
        $query = "SELECT
                    COUNT(*) CONTADOR
                  FROM
                    ITEMPEDEXOL
                  WHERE
                    NRPEDEXOL = :P_NRPEDEXOL
                    AND DTPROGENPEOL = TO_DATE(:P_DTPROGENPEOL, 'DD/MM/YYYY')
                    AND CDMARCA = :P_CDMARCA
                    AND CDPRODUTO = :P_CDPRODUTO";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function insereItemPedidoOL($row) {
        $param = array();
        $param["CDFILDEST"]         = $row["CDFILDEST"];
        $param["CDFILLANC"]         = $row["CDFILLANC"];
        $param["NRAUTOFORN"]        = $row["NRPEDEXOL"];
        $param["CDPRODUTO"]         = $row["CDPRODUTO"];
        $param["QTPROGENTAF"]       = $row["QTPROGENPEOL"];
        $param["QTEFETENTRAF"]      = null;
        $param["DTPROGENTAF"]       = $row["DTPROGENPEOL"];
        $param["DTEFETENTAF"]       = null;
        $param["CDMARCA"]           = $row["CDMARCA"];
        $param["CDLOTENVITOL"]      = $row["CDLOTENVITOL"];
        $param["IDBAIXITMAF"]       = null;
        $param["IDBAIXITAFS"]       = null;
        $param["TXOBSITMAF"]        = isset($row["TXOBSITMPEOL"]) ? $row["TXOBSITMPEOL"] : null;
        $param["CDOPERBAIXA"]       = null;
        $param["DTBAIXITMAF"]       = null;
        $param["QTDORIGINAL"]       = $row["QTPROGENPEOL"];
        $param["CDPRODESTO"]        = $row["CDPRODESTO"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDARVPROD"]       = $row["CDARVPROD"];
        $param["CDRASTRITM"]        = null;
        $param["VRPREUNITPED"]      = null;
        
        $query = "INSERT INTO ITEMPEDEXOL
                      (CDFILDEST,    CDFILLANC,    NRPEDEXOL,    CDPRODUTO,     QTPROGENPEOL,   QTEFETENPEOL,
                       DTPROGENPEOL, DTEFETENPEOL, CDMARCA,      CDLOTENVITOL,  IDBAIXENPEOL,   IDBAIXAOLSAI,
                       TXOBSITMPEOL, CDOPERBAIXA,  DTBAIXTMPEOL, QTORIGPRODPED, CDPRODORIGPEDOL,
                       DTPROGEMBARQ, CDRASTRITM, VRPREUNITPED)
                    SELECT :CDFILDEST,   :CDFILLANC,   :NRAUTOFORN,  :CDPRODUTO,    :QTPROGENTAF, :QTEFETENTRAF,
                           :DTPROGENTAF, :DTEFETENTAF, :CDMARCA,     :CDLOTENVITOL, :IDBAIXITMAF, :IDBAIXITAFS,
                           :TXOBSITMAF,  :CDOPERBAIXA, :DTBAIXITMAF, :QTDORIGINAL,  :CDPRODESTO,
                           CALCULA_DATAEMBARQUE_OL(:P_CDFILIOPELOG, :CDFILDEST, :P_CDARVPROD, :DTPROGENTAF), :CDRASTRITM,
                           :VRPREUNITPED
                      FROM PEDIDOEXOL P
                     WHERE P.CDFILDEST = :CDFILDEST
                       AND P.CDFILLANC = :CDFILLANC
                       AND P.NRPEDEXOL = :NRAUTOFORN
                       AND NVL(P.IDBAIXEXSAI, '1') <> '2'";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function excluiItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "DELETE FROM ITEMPEDEXOL
                    WHERE
                      CDFILDEST = :OLD_CDFILDEST AND
                      CDFILLANC = :OLD_CDFILLANC AND
                      NRPEDEXOL = :OLD_NRAUTOFORN AND
                      CDPRODUTO = :OLD_CDPRODUTO AND
                      DTPROGENPEOL = :OLD_DTPROGENTAF AND
                      CDMARCA = :OLD_CDMARCA AND
                      NVL(IDBAIXAOLSAI, '1') <> '2'";
                      
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraDataEntregaItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["DTPROGENTAF"]       = $row["NEW_DTPROGENPEOL"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["OLD_DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        DTPROGENPEOL = :DTPROGENTAF
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraMarcaItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["CDMARCA"]           = $row["NEW_CDMARCA"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["OLD_CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        CDMARCA = :CDMARCA
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraProdutoItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["CDPRODUTO"]         = $row["NEW_CDPRODUTO"];
        $param["CDPRODESTO"]        = $row["NEW_CDPRODESTO"];
        $param["QTPROGENPEOL"]      = $row["NEW_QTPROGENPEOL"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["OLD_CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        CDPRODUTO = :CDPRODUTO,
                        CDPRODORIGPEDOL = :CDPRODESTO,
                        QTPROGENPEOL = :QTPROGENPEOL
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function alteraQuantidadeItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["QTPROGENPEOL"]      = $row["NEW_QTPROGENPEOL"];
        $param["OLD_CDFILDEST"]     = $CDFILDEST[0];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $row["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $row["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $row["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                    SET
                        QTPROGENPEOL = :QTPROGENPEOL
                    WHERE
                            CDFILDEST = :OLD_CDFILDEST
                        AND CDFILLANC = :OLD_CDFILLANC
                        AND NRPEDEXOL = :OLD_NRAUTOFORN
                        AND CDPRODUTO = :OLD_CDPRODUTO
                        AND DTPROGENPEOL = :OLD_DTPROGENTAF
                        AND CDMARCA = :OLD_CDMARCA
                        AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function pegaUltimoNumeroPedido($CDFILLANC, $CDFILDEST) {
        $param = array();
        $param["P_CDFILLANC"] = $CDFILLANC;
        $param["P_CDFILDEST"] = $CDFILDEST;
        
        $query = "SELECT NRPEDEXOL FROM PEDIDOEXOL
                    WHERE ROWNUM = 1 
                    AND CDFILDEST = :P_CDFILDEST 
                    AND CDFILLANC = :P_CDFILLANC 
                    ORDER BY NRPEDEXOL DESC";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(empty($return)) {
                $return = $this->procedureGeraNovoCodigo("PEDIDOEXOL");
            } else {
                $return = $return[0]["NRPEDEXOL"];
            }
            return $return;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function inclusaoDePedidoParaLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $row["NEW_NRPEDEXOL"] = self::pegaUltimoNumeroPedido($row["CDFILLANC"], $CDFILDEST[0]);
        $row["NEW_NRPEDEXOL"] = intval($row["NEW_NRPEDEXOL"]);
        $row["NEW_NRPEDEXOL"]++;
        $row["NEW_NRPEDEXOL"] = str_pad($row["NEW_NRPEDEXOL"], 10, 0, STR_PAD_LEFT);
        
        $param = array();
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_NRPEDEXOL_NEW"]   = $row["NEW_NRPEDEXOL"];
        
        $query = "INSERT INTO PEDIDOEXOL
                  (CDFILDEST, CDFILLANC, NRPEDEXOL, DTPEDEXOL,
                   IDEMITEXOL, IDTPFRETEXOL, CDEMPRESA, CDTIPCOB,
                   IDTPJTREXOL,  NRINSJTREXOL, CDTPCTPAGAR, CDFILIOPELOG, IDTIPOEXOL, IDPEDEXOLEXTR)
                  (SELECT P.CDFILDEST, P.CDFILLANC, :P_NRPEDEXOL_NEW, TRUNC(SYSDATE),
                          P.IDEMITEXOL, P.IDTPFRETEXOL, P.CDEMPRESA, P.CDTIPCOB,
                          P.IDTPJTREXOL,  P.NRINSJTREXOL, P.CDTPCTPAGAR, P.CDFILIOPELOG, P.IDTIPOEXOL, P.IDPEDEXOLEXTR
                    FROM PEDIDOEXOL P
                   WHERE P.CDFILDEST    = :P_CDFILDEST
                     AND P.CDFILLANC    = :P_CDFILLANC
                     AND P.NRPEDEXOL    = :P_NRPEDEXOL
                     AND NVL(P.IDBAIXEXSAI, '1') <> '2')";
                     
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function incluiItemPedidoLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $row["NEW_NRPEDEXOL"] = self::pegaUltimoNumeroPedido($row["CDFILLANC"], $CDFILDEST[0]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDOPERADOR"]      = $this->environment->getCdOperador();
        $param["P_CDLOTENVITOL"]    = $row["NEW_CDLOTENVITOL"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_NRPEDEXOL_NEW"]   = $row["NEW_NRPEDEXOL"];
        
        $query = "INSERT INTO ITEMPEDEXOL
                      (CDFILDEST, CDFILLANC, NRPEDEXOL, CDPRODUTO, QTPROGENPEOL, QTEFETENPEOL,
                       DTPROGENPEOL, DTEFETENPEOL,  CDMARCA, CDLOTENVITOL,
                       IDBAIXENPEOL, IDBAIXAOLSAI, TXOBSITMPEOL, CDOPERBAIXA, DTBAIXTMPEOL, DTPROGEMBARQ, CDOPERBLOQITM, CDRASTRITM)
                      (SELECT I.CDFILDEST, I.CDFILLANC, :P_NRPEDEXOL_NEW, I.CDPRODUTO, I.QTPROGENPEOL, I.QTEFETENPEOL,
                              I.DTPROGENPEOL, I.DTEFETENPEOL, I.CDMARCA, :P_CDLOTENVITOL,
                              I.IDBAIXENPEOL, I.IDBAIXAOLSAI, I.TXOBSITMPEOL, I.CDOPERBAIXA,
                              I.DTBAIXTMPEOL, I.DTPROGEMBARQ, :P_CDOPERADOR, I.CDRASTRITM
                        FROM ITEMPEDEXOL I
                       WHERE I.CDFILDEST    = :P_CDFILDEST
                         AND I.CDFILLANC    = :P_CDFILLANC
                         AND I.NRPEDEXOL    = :P_NRPEDEXOL
                         AND I.CDPRODUTO    = :P_CDPRODUTO
                         AND DTPROGENPEOL   = :P_DTPROGENPEOL
                         AND I.CDMARCA      = :P_CDMARCA)";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function ajusteItemPedidoLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                   SET IDITEMAJUSTE = 'S'
                 WHERE CDFILDEST    = :P_CDFILDEST AND
                  CDFILLANC    = :P_CDFILLANC AND
                  NRPEDEXOL    = :P_NRPEDEXOL AND
                  CDPRODUTO    = :P_CDPRODUTO AND
                  DTPROGENPEOL = :P_DTPROGENPEOL AND
                  CDMARCA      = :P_CDMARCA";
                  
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function excluiReservaItemPedidoLote($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        
        $query = "DELETE FROM RESERVAPEDEXOL
                     WHERE CDFILDEST     = :P_CDFILDEST
                       AND CDFILLANC     = :P_CDFILLANC
                       AND NRPEDEXOL     = :P_NRPEDEXOL
                       AND CDPRODUTO     = :P_CDPRODUTO
                       AND CDMARCA       = :P_CDMARCA
                       AND DTPROGENPEOL  = :P_DTPROGENPEOL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function verificaOLQueAtendeFilial($row) {
        $param = array();
        $param["P_CDFILIOPELOG"] = $row["CDFILIOPELOG"];
        
        $query = "SELECT NVL(MAX(CDFILIAL), :P_CDFILIOPELOG) CDFILIAL
                     FROM FILIAL
                    WHERE CDFILIOLAGETERC = :P_CDFILIOPELOG";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            $return = $return[0]["CDFILIAL"];
            return $return;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function posicaoEstoqueItemOL($CDOLFILIAL, $row) {
        $param = array();
        $param["P_CDFILIOPELOG"]    = $CDOLFILIAL;
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        
        $query = "SELECT CALC_ESTOQ_CONS(:P_CDFILIOPELOG, Q.CDPRODUTO, Q.DTPOSESTCON) QUANT_ESTOQUE
                  FROM (SELECT P.CDPRODUTO, NVL(MAX(DTPOSESTCON), SYSDATE) AS DTPOSESTCON
                          FROM (SELECT DECODE (R.CDPRODARVOL, P.CDPRODESTO, R.CDPRODARVOL, PR.CDPRODUTO) AS CDPRODUTO
                                  FROM RELACPRODOL R, PRODUTO P, PRODUTO PR
                                 WHERE P.CDPRODUTO   = R.CDPRODARVOL
                                   AND P.CDPRODESTO  = PR.CDPRODUTO
                                   AND R.CDPRARVORIG = :P_CDPRODUTO
                                   AND R.CDMARCA     = :P_CDMARCA) P, POSESTCONS E
                         WHERE E.CDFILIAL  = :P_CDFILIOPELOG
                           AND E.CDPRODUTO = P.CDPRODUTO
                         GROUP BY P.CDPRODUTO) Q";
                         
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0]["QUANT_ESTOQUE"];
            else
                return null;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function verificaReservaDemaisPedidos($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_NRAUTOFORN"]      = $row["NRPEDEXOL"];
        $param["P_DTPROGENTAF"]     = $row["DTPROGENPEOL"];
        
        $query = "SELECT SUM(QTRESERVESTOL) ESTOQ_RESERVADO
                      FROM RESERVAPEDEXOL
                     WHERE CDPRODUTO    =  :P_CDPRODUTO
                       AND CDMARCA      =  :P_CDMARCA
                       AND CDFILRESOL   =  :P_CDFILIOPELOG
                       AND (CDFILDEST   <> :P_CDFILDEST
                        OR CDFILLANC    <> :P_CDFILLANC
                        OR NRPEDEXOL    <> :P_NRAUTOFORN
                        OR DTPROGENPEOL <> :P_DTPROGENTAF)";
                         
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0]["ESTOQ_RESERVADO"];
            else
                return null;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function incluiReservaItemPedidoOL($row, $QTRESERVESTOL) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_QTRESERVESTOL"]   = $QTRESERVESTOL;
        
        $query = "INSERT INTO RESERVAPEDEXOL (CDFILDEST, CDFILLANC, NRPEDEXOL, CDPRODUTO, CDMARCA,
                                                DTPROGENPEOL, QTRESERVESTOL, CDFILRESOL, DTRESEMBARQ)
                    SELECT :P_CDFILDEST, :P_CDFILLANC, :P_NRPEDEXOL, :P_CDPRODUTO, :P_CDMARCA,
                           :P_DTPROGENPEOL, :P_QTRESERVESTOL, :P_CDFILIOPELOG,
                           CALCULA_DATAEMBARQUE_OL(:P_CDFILIOPELOG, :P_CDFILDEST,
                           :P_CDPRODUTO,  :P_DTPROGENPEOL)
                      FROM DUAL
                     WHERE NOT EXISTS (SELECT 1
                                         FROM RESERVAPEDEXOL R
                                        WHERE R.CDFILDEST    = :P_CDFILDEST
                                          AND R.CDFILLANC    = :P_CDFILLANC
                                          AND R.NRPEDEXOL    = :P_NRPEDEXOL
                                          AND R.CDPRODUTO    = :P_CDPRODUTO
                                          AND R.CDMARCA      = :P_CDMARCA
                                          AND R.DTPROGENPEOL = :P_DTPROGENPEOL)";
                                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaSituacaoItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["QTPROGENPEOL"]      = $row["QTPROGENPEOL"];
        $param["IDBAIXAOLSAI"]      = $row["NEW_IDBAIXAOLSAI"];
        $param["CDOPERBAIXA"]       = $row["NEW_CDOPERBAIXA"];
        $param["DTBAIXTMPEOL"]      = $row["NEW_DTBAIXTMPEOL"];
        $param["QTORIGPRODPED"]     = $row["QTORIGPRODPED"];
        $param["CDFILDEST"]         = $CDFILDEST[0];
        $param["CDFILLANC"]         = $row["CDFILLANC"];
        $param["NRPEDEXOL"]         = $row["NRPEDEXOL"];
        $param["CDPRODUTO"]         = $row["CDPRODUTO"];
        $param["CDMARCA"]           = $row["CDMARCA"];
        $param["DTPROGENPEOL"]      = $row["DTPROGENPEOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                       SET QTPROGENPEOL  = NVL(:QTPROGENPEOL, QTPROGENPEOL),
                           IDBAIXAOLSAI  = NVL(:IDBAIXAOLSAI, IDBAIXAOLSAI),
                           CDOPERBAIXA   = NVL(:CDOPERBAIXA ,  CDOPERBAIXA),
                           DTBAIXTMPEOL  = NVL(:DTBAIXTMPEOL, DTBAIXTMPEOL),
                           QTORIGPRODPED = :QTORIGPRODPED
                     WHERE CDFILDEST     = :CDFILDEST
                       AND CDFILLANC     = :CDFILLANC
                       AND NRPEDEXOL     = :NRPEDEXOL
                       AND CDPRODUTO     = :CDPRODUTO
                       AND CDMARCA       = :CDMARCA
                       AND DTPROGENPEOL  = :DTPROGENPEOL
                       AND NVL(IDBAIXAOLSAI, '1') <> '2'";
                  
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function checkOperBlockUnblockPer() {
        $param = array();
        $param["P_CDOPERADOR"] = $this->environment->getCdOperador();
        
        $query = "SELECT O.CDOPERADOR 
                      FROM OPERGRUPOP O, AUTGRUPOP A
                     WHERE O.CDOPERADOR  = :P_CDOPERADOR
                       AND O.CDGRUPOPER  = A.CDGRUPOPER
                       AND A.IDAUTGRUPOP = 'BLOL'";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function desbloqueioBloqueioItemPedidoOL($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_IDITMPEDOLBLOQ"]  = $row["NEW_IDITMPEDOLBLOQ"];
        $param["P_DTBLOQITMPEDOL"]  = $row["NEW_DTBLOQITMPEDOL"];
        $param["P_CDPRODUTO"]       = $row["CDPRODUTO"];
        $param["P_CDOPERADOR"]      = $row["NEW_CDOPERADOR"];
        $param["P_CDMARCA"]         = $row["CDMARCA"];
        $param["P_CDFILLANC"]       = $row["CDFILLANC"];
        $param["P_DTPROGENPEOL"]    = $row["DTPROGENPEOL"];
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_NRPEDEXOL"]       = $row["NRPEDEXOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                       SET IDITMPEDOLBLOQ = :P_IDITMPEDOLBLOQ,
                           DTBLOQITMPEDOL = :P_DTBLOQITMPEDOL,
                           CDOPERBLOQITM  = :P_CDOPERADOR
                      WHERE CDFILDEST     = :P_CDFILDEST
                        AND CDFILLANC     = :P_CDFILLANC
                        AND NRPEDEXOL     = :P_NRPEDEXOL
                        AND CDMARCA       = :P_CDMARCA
                        AND CDPRODUTO     = :P_CDPRODUTO
                        AND DTPROGENPEOL  = :P_DTPROGENPEOL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getAdqParametros($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDFILDEST"] = $CDFILDEST[0];
        
        $query = "SELECT PARASUPRI.IDGERAITEMSOLI, PARCMPSUP.IDUTAPROVSOL
                    FROM PARASUPRI, FILIAL, PARCMPSUP
                   WHERE PARASUPRI.CDFILIAL = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                     AND FILIAL.CDFILIAL = :P_CDFILDEST
                     AND FILIAL.CDFILIAL = PARCMPSUP.CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0];
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getAdqItemSoliEd($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDFILDEST"]   = $CDFILDEST[0];
        $param["P_DTPROGENTR"]  = $row["DTPROGENPEOL"];
        $param["P_CDPRODESTO"]  = $row["CDPRODESTO"];
        
        $query = "SELECT ITEMSOLI.CDFILSOLI, ITEMSOLI.CDFILLAN, ITEMSOLI.NRSOLICMP
                      , ITEMSOLI.CDPRODUTO, ITEMSOLI.DTPROGENTR
                      FROM ITEMSOLI
                     WHERE ITEMSOLI.CDFILSOLI  = :P_CDFILDEST
                       AND ITEMSOLI.DTPROGENTR = TO_DATE(:P_DTPROGENTR, 'DD/MM/YYYY')
                       AND ITEMSOLI.CDPRODUTO IN (SELECT PRODUTO.CDPRODUTO
                                                    FROM PRODUTO
                                                   WHERE CDPRODESTO = :P_CDPRODESTO)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0];
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getAdqPesqItemSoli($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDFILDEST"]   = $CDFILDEST[0];
        $param["P_CDFILLANC"]   = $row["CDFILLANC"];
        $param["P_CDPRODESTO"]  = $row["CDPRODESTO"];
        
        $query = "SELECT
                        MAX(ITEMSOLI.NRSOLICMP) NRSOLICMP,
                        ITEMSOLI.CDFILLAN,
                        ITEMSOLI.CDPRODUTO
                    FROM
                        ITEMSOLI,
                        PRODUTO
                    WHERE
                            ITEMSOLI.CDFILSOLI  = :P_CDFILDEST
                        AND ITEMSOLI.CDPRODUTO  = PRODUTO.CDPRODUTO
                        AND PRODUTO.CDPRODESTO  = :P_CDPRODESTO
                        AND ITEMSOLI.CDFILLAN   = :P_CDFILLANC
                        AND ITEMSOLI.DTPROGENTR = (
                            SELECT
                                MAX(ITEMSOLI.DTPROGENTR)
                            FROM
                                ITEMSOLI,
                                PRODUTO
                            WHERE
                                    ITEMSOLI.CDFILSOLI = :P_CDFILDEST
                                AND ITEMSOLI.CDPRODUTO = PRODUTO.CDPRODUTO
                                AND PRODUTO.CDPRODESTO = :P_CDPRODESTO
                                AND ITEMSOLI.CDFILLAN  = :P_CDFILLANC
                        )
                    GROUP BY
                        ITEMSOLI.CDFILLAN,
                        ITEMSOLI.CDPRODUTO";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0];
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getLastNRSOLICMP($CDFILLANC, $CDFILDEST) {
        $param = array();
        $param["P_CDFILLANC"] = $CDFILLANC;
        $param["P_CDFILDEST"] = $CDFILDEST;
        
        $query = "SELECT
                        NRSOLICMP
                    FROM
                        SOLICITA
                    WHERE
                            ROWNUM = 1
                        AND CDFILSOLI = :P_CDFILDEST
                        AND CDFILLANCA = :P_CDFILLANC
                    ORDER BY
                        NRSOLICMP DESC";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            $return  = $return[0]["NRSOLICMP"];
            return $return;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsSolicitaItemPedidoED($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $NRSOLICMP = self::getLastNRSOLICMP($row["CDFILLANC"], $CDFILDEST[0]);
        $NRSOLICMP = intval($NRSOLICMP);
        $NRSOLICMP++;
        $NRSOLICMP = str_pad($NRSOLICMP, 10, 0, STR_PAD_LEFT);
        
        $param = array();
        $param["P_CDFILSOLI"]   = $CDFILDEST[0];
        $param["P_CDFILLANC"]   = $row["CDFILLANC"];
        $param["P_CDOPERADOR"]  = $row["CDOPERADOR"];
        $param["P_NRSOLICMP"]   = $NRSOLICMP;
        
        $query = "INSERT INTO SOLICITA (
                    CDFILLANCA,
                    CDFILSOLI,
                    NRSOLICMP,
                    DTSOLCMP,
                    IDTIPLAN,
                    CDFILCENT,
                    CDOPERLANC )
                    ( SELECT
                        :P_CDFILLANC,
                        :P_CDFILSOLI,
                        :P_NRSOLICMP,
                        TRUNC(SYSDATE),
                        'M',
                        NVL(F.CDFILCENTCMP, F.CDFILIAL),
                        :P_CDOPERADOR
                    FROM
                        FILIAL F
                    WHERE
                            F.CDFILIAL = :P_CDFILSOLI
                        AND NOT EXISTS (
                            SELECT
                                1
                            FROM
                                SOLICITA S
                            WHERE
                                    S.CDFILLANCA = :P_CDFILLANC
                                 AND S.CDFILSOLI = :P_CDFILSOLI
                                 AND S.NRSOLICMP = :P_NRSOLICMP ))";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsSolPontoaPontoItemPedidoED($row, $adqParametros, $adqPesqItemSoli) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $NRSOLICMP = self::getLastNRSOLICMP($row["CDFILLANC"], $CDFILDEST[0]);
        
        $param = array();
        $param["P_CDFILSOLI"]   = $CDFILDEST[0];
        $param["P_CDFILLANC"]   = is_null($adqPesqItemSoli) ? $row["CDFILLANC"] : $adqPesqItemSoli["CDFILLAN"];
        $param["P_NRSOLICMP"]   = is_null($adqPesqItemSoli) ? $NRSOLICMP : $adqPesqItemSoli["NRSOLICMP"];
        $param["P_DTPROGENTR"]  = $row["DTPROGENPEOL"];
        $param["P_CDPRODUTO"]   = $row["CDPRODESTO"];
        $param["P_QTSOLIPROD"]  = ($row["QTPROGENPEOL"] * $row["VRFATOCONV"]);
        $param["P_IDSOLIAPROV"] = $adqParametros["IDUTAPROVSOL"] == 'N' ? 'S' : 'N';
        $param["P_CDRASTRITM"]  = null;
        
        $query = "INSERT INTO ITEMSOLI (CDFILSOLI, CDFILLAN, NRSOLICMP, DTPROGENTR, CDPRODUTO, QTSOLIPROD, IDSOLIAPROV, IDITMSOLEXTRA, CDFILIOPELOG, QTSOLIPRODORIG, CDPRODUTORIG, CDRASTRITM)
                    VALUES(:P_CDFILSOLI, :P_CDFILLANC, :P_NRSOLICMP, :P_DTPROGENTR, :P_CDPRODUTO, :P_QTSOLIPROD, :P_IDSOLIAPROV, 'N', NULL, :P_QTSOLIPROD, :P_CDPRODUTO, :P_CDRASTRITM)";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsMovopSolItemPedidoED($row, $adqPesqItemSoli) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $NRSOLICMP = self::getLastNRSOLICMP($row["CDFILLANC"], $CDFILDEST[0]);
        
        $param = array();
        $param["P_CDFILSOLMOV"]     = $CDFILDEST[0];
        $param["P_CDFILLANMOV"]     = is_null($adqPesqItemSoli) ? $row["CDFILLANC"] : $adqPesqItemSoli["CDFILLAN"];
        $param["P_NRSOLIMOV"]       = is_null($adqPesqItemSoli) ? $NRSOLICMP : $adqPesqItemSoli["NRSOLICMP"];
        $param["P_CDPRODUTO"]       = $row["CDPRODESTO"];
        $param["P_DTENTROPER"]      = $row["DTPROGENPEOL"];
        $param["P_CDOPERADOR"]      = $row["CDOPERADOR"];
        $param["P_QTANTOPER"]       = 0;
        $param["P_QTDEPOPER"]       = $row["QTPROGENPEOL"];
        $param["P_DSJUSTOPER"]      = "Alteração do pedido para Entrega Direta";
        
        $query = "INSERT INTO MOVOPSOL
                        (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV,
                        CDPRODUTO, DTENTROPER, DTOPERSOL,
                        CDOPERADOR, QTANTOPER, QTDEPOPER,
                        DSJUSTOPER)
                    VALUES
                        (:P_CDFILSOLMOV, :P_CDFILLANMOV, :P_NRSOLIMOV,
                        :P_CDPRODUTO, :P_DTENTROPER, SYSDATE,
                        :P_CDOPERADOR, :P_QTANTOPER, :P_QTDEPOPER,
                        :P_DSJUSTOPER)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmSolPontoaPonto2ItemPedidoED($row, $adqItemSoliEd) {
        $param = array();
        $param["P_QTSOLIPROD"] = ($row["QTPROGENPEOL"] * $row["VRFATOCONV"]);
        $param["P_CDFILDEST"] = $adqItemSoliEd["CDFILSOLI"];
        $param["P_CDFILLAN"] = $adqItemSoliEd["CDFILLAN"];
        $param["P_NRSOLICMP"] = $adqItemSoliEd["NRSOLICMP"];
        $param["P_DTPROGENTR"] = $adqItemSoliEd["DTPROGENTR"];
        $param["P_CDPRODESTO"] = $adqItemSoliEd["CDPRODUTO"];
        
        $query = "UPDATE ITEMSOLI
                        SET QTSOLIPROD   = CASE NVL(NRSEQUCONSOL, 'S')
                        WHEN 'S'
                        THEN QTSOLIPROD + :P_QTSOLIPROD
                        ELSE QTSOLIPROD
                        END,
                        IDPEDIDOOL   = 'N',
                        CDFILIOPELOG = NULL,
                        NRSEQUCONSOL = NULL
                      WHERE ITEMSOLI.CDFILSOLI    = :P_CDFILDEST
                        AND ITEMSOLI.CDFILLAN     = :P_CDFILLAN
                        AND ITEMSOLI.NRSOLICMP    = :P_NRSOLICMP
                        AND ITEMSOLI.DTPROGENTR   = :P_DTPROGENTR
                        AND ITEMSOLI.CDPRODUTO    = :P_CDPRODESTO";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmSolPontoAPontoItemPedidoED($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDFILDEST"]   = $CDFILDEST[0];
        $param["P_DTPROGENTR"]  = $row["DTPROGENTR"];
        $param["P_CDPRODESTO"]  = $row["CDPRODESTO"];
         
        $query = "UPDATE ITEMSOLI
                     SET IDPEDIDOOL   = 'N',
                         CDFILIOPELOG = NULL,
                         NRSEQUCONSOL = NULL
                   WHERE ITEMSOLI.CDFILSOLI    = :P_CDFILDEST
                     AND ITEMSOLI.DTPROGENTR   = :P_DTPROGENTR
                     AND ITEMSOLI.CDFILIOPELOG IS NOT NULL
                     AND ITEMSOLI.IDPEDIDOOL   = 'S'
                     AND ITEMSOLI.NRSEQUCONSOL IS NOT NULL
                     AND ITEMSOLI.CDFILIOPELOG <> ITEMSOLI.CDFILSOLI
                     AND ITEMSOLI.CDPRODUTO IN (SELECT PRODUTO.CDPRODUTO
                                                  FROM PRODUTO
                                                 WHERE CDPRODESTO = :P_CDPRODESTO)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function carregaDadosPedido($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDFILLANC"] = $row["CDFILLANC"];
        $param["P_CDFILDEST"] = $CDFILDEST[0];
        $param["P_NRPEDEXOL"] = $row["NRPEDEXOL"];
        
        $query = "SELECT
                    	P.CDFILDEST,
                    	F1.NMFILIAL    NMFILDEST,
                    	P.CDFILLANC,
                    	F2.NMFILIAL    NMFILLANC,
                    	P.CDFILIOPELOG,
                    	F3.NMFILIAL    NMFILIOPELOG,
                    	P.NRPEDEXOL,
                    	P.DTPEDEXOL,
                    	P.IDBAIXEXOL,
                    	P.IDBAIXEXSAI,
                    	P.CDEMPRESA,
                    	P.IDPEDEXOLEXTR,
                    	P.IDEMITEXOL,
                    	P.CDTIPCOB,
                    	TC.NMTIPOCOBR,
                    	P.IDTPFRETEXOL,
                    	P.CDTPCTPAGAR,
                    	TP.DSTPCTPAGAR,
                    	P.NRINSJTREXOL,
                    	P.IDTPJTREXOL,
                    	T.NMRAZSOCTRAN
                      FROM
                    	PEDIDOEXOL   P,
                    	FILIAL       F1,
                    	FILIAL       F2,
                    	FILIAL       F3,
                    	TIPOCOBR     TC,
                    	TIPOCTPAGAR  TP,
                    	TRANSPORT    T
                     WHERE
                    	    F1.CDFILIAL = P.CDFILDEST
                    	   AND F2.CDFILIAL             = P.CDFILLANC
                    	   AND F3.CDFILIAL             = P.CDFILIOPELOG
                    	   AND TC.CDTIPCOB (+)         = P.CDTIPCOB
                    	   AND TP.CDTPCTPAGAR (+)      = P.CDTPCTPAGAR
                    	   AND T.IDTPIJURTRAN (+)      = P.IDTPJTREXOL
                    	   AND T.NRINSJURTRAN (+)      = P.NRINSJTREXOL
                    	   AND P.CDFILDEST             = :P_CDFILDEST
                    	   AND P.CDFILLANC             = :P_CDFILLANC
                    	   AND NRPEDEXOL               = :P_NRPEDEXOL";
                    
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            return $return[0];
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function salvaDadosPedido($row) {
        $param = array();
        $param["DTAUTOFORN"]        = $row["DTPEDEXOL"];
        $param["IDTIPOEMITAF"]      = $row["IDTIPOEMITAF"];
        $param["IDTIPOFRETAF"]      = $row["IDTIPOFRETAF"];
        $param["CDEMPRESA"]         = $row["CDEMPRESA"];
        $param["CDTIPCOB"]          = $row["CDTIPCOB"];
        $param["IDTPIJURTRAN"]      = $row["IDTPIJURTRAN"];
        $param["NRINSJURTRAN"]      = $row["NRINSJURTRAN"];
        $param["IDBAIXAAF"]         = isset($row["NEW_IDBAIXEXOL"]) ? $row["NEW_IDBAIXEXOL"] == 0 ? null : $row["NEW_IDBAIXEXOL"] : $row["IDBAIXEXOL"];
        $param["IDBAIXAFSAI"]       = $row["IDBAIXEXSAI"];
        $param["CDTPCTPAGAR"]       = $row["CDTPCTPAGAR"];
        $param["IDPEDEXOLEXTR"]     = $row["IDPEDEXOLEXTR"];
        $param["IDUTPEDINTEGRAL"]   = 'N';
        $param["OLD_CDFILDEST"]     = $row["CDFILDEST"];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        
        $query = "UPDATE PEDIDOEXOL
                       SET
                    	DTPEDEXOL = :DTAUTOFORN,
                    	IDEMITEXOL = :IDTIPOEMITAF,
                    	IDTPFRETEXOL = :IDTIPOFRETAF,
                    	CDEMPRESA = :CDEMPRESA,
                    	CDTIPCOB = :CDTIPCOB,
                    	IDTPJTREXOL = :IDTPIJURTRAN,
                    	NRINSJTREXOL = :NRINSJURTRAN,
                    	IDBAIXEXOL = :IDBAIXAAF,
                    	IDBAIXEXSAI = :IDBAIXAFSAI,
                    	CDTPCTPAGAR = :CDTPCTPAGAR,
                    	IDPEDEXOLEXTR = :IDPEDEXOLEXTR,
                    	USR_IDUTPEDINTEGRAL = :IDUTPEDINTEGRAL
                     WHERE
                    	    CDFILDEST = :OLD_CDFILDEST
                    	   AND CDFILLANC = :OLD_CDFILLANC
                    	   AND NRPEDEXOL = :OLD_NRAUTOFORN";
                        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function selecionaItensDoPedido($row) {
        $param = array();
        $param["P_NRPEDEXOL"] = $row["NRPEDEXOL"];
        $param["P_CDFILDEST"] = $row["CDFILDEST"];
        $param["P_CDFILLANC"] = $row["CDFILLANC"];
        
        $query = "SELECT
                    	CDFILDEST,
                    	CDFILLANC,
                    	IDBAIXAOLSAI,
                    	IDBAIXENPEOL,
                    	QTPROGENPEOL,
                    	QTEFETENPEOL,
                    	DTEFETENPEOL,
                    	TXOBSITMPEOL,
                    	QTORIGPRODPED,
                    	VRPREUNITPED,
                    	NRPEDEXOL,
                    	CDPRODUTO,
                    	TO_CHAR(DTPROGENPEOL, 'DD/MM/YYYY') DTPROGENPEOL,
                    	CDMARCA,
                    	DTBAIXTMPEOL
                      FROM
                    	ITEMPEDEXOL
                     WHERE
                    	    NRPEDEXOL = :P_NRPEDEXOL
                    	   AND CDFILDEST  = :P_CDFILDEST
                    	   AND CDFILLANC  = :P_CDFILLANC";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaSitItemViaPedido($item) {
        $param = array();
        $param["QTPROGENTAF"]       = $item["QTPROGENPEOL"];
        $param["QTEFETENTRAF"]      = $item["QTEFETENPEOL"];
        $param["DTEFETENTAF"]       = $item["DTEFETENPEOL"];
        $param["IDBAIXITMAF"]       = $item["NEW_IDBAIXENPEOL"] == 0 ? null : $item["NEW_IDBAIXENPEOL"];
        $param["IDBAIXITAFS"]       = $item["IDBAIXAOLSAI"];
        $param["TXOBSITMAF"]        = $item["TXOBSITMPEOL"];
        $param["CDOPERBAIXA"]       = $this->environment->getCdOperador();
        $param["DTBAIXITMAF"]       = isset($item["NEW_DTBAIXTMPEOL"]) ? : $item["DTBAIXTMPEOL"] ;
        $param["QTDORIGINAL"]       = $item["QTORIGPRODPED"];
        $param["VRPREUNITPED"]      = $item["VRPREUNITPED"];
        $param["OLD_CDFILDEST"]     = $item["CDFILDEST"];
        $param["OLD_CDFILLANC"]     = $item["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $item["NRPEDEXOL"];
        $param["OLD_CDPRODUTO"]     = $item["CDPRODUTO"];
        $param["OLD_DTPROGENTAF"]   = $item["DTPROGENPEOL"];
        $param["OLD_CDMARCA"]       = $item["CDMARCA"];
        
        $query = "UPDATE ITEMPEDEXOL
                       SET
                    	QTPROGENPEOL = :QTPROGENTAF,
                    	QTEFETENPEOL = :QTEFETENTRAF,
                    	DTEFETENPEOL = :DTEFETENTAF,
                    	IDBAIXENPEOL = :IDBAIXITMAF,
                    	IDBAIXAOLSAI = :IDBAIXITAFS,
                    	TXOBSITMPEOL = :TXOBSITMAF,
                    	CDOPERBAIXA = :CDOPERBAIXA,
                    	DTBAIXTMPEOL = :DTBAIXITMAF,
                    	QTORIGPRODPED = :QTDORIGINAL,
                    	VRPREUNITPED = :VRPREUNITPED
                     WHERE
                    	    CDFILDEST = :OLD_CDFILDEST
                    	   AND CDFILLANC   = :OLD_CDFILLANC
                    	   AND NRPEDEXOL   = :OLD_NRAUTOFORN
                    	   AND CDPRODUTO   = :OLD_CDPRODUTO
                    	   AND DTPROGENPEOL = :OLD_DTPROGENTAF
                    	   AND CDMARCA     = :OLD_CDMARCA
                    	   AND NVL(IDBAIXAOLSAI, '1') <> '2'";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function exclusaoItensPedido($row) {
        $param = array();
        $param["P_CDFILDEST"] = $row["CDFILDEST"];
        $param["P_CDFILLANC"] = $row["CDFILLANC"];
        $param["P_NRPEDEXOL"] = $row["NRPEDEXOL"];
        
        $query = "DELETE FROM ITEMPEDEXOL
                    WHERE
                    CDFILDEST = :P_CDFILDEST AND
                    CDFILLANC = :P_CDFILLANC AND 
                    NRPEDEXOL = :P_NRPEDEXOL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function exclusaoPedido($row) {
        $param = array();
        $param["OLD_CDFILDEST"]     = $row["CDFILDEST"];
        $param["OLD_CDFILLANC"]     = $row["CDFILLANC"];
        $param["OLD_NRAUTOFORN"]    = $row["NRPEDEXOL"];
        
        $query = "DELETE FROM PEDIDOEXOL
                    WHERE
                      CDFILDEST = :OLD_CDFILDEST AND
                      CDFILLANC = :OLD_CDFILLANC AND
                      NRPEDEXOL = :OLD_NRAUTOFORN";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function inclusaoPedido($row) {
        $row["NEW_NRPEDEXOL"] = self::pegaUltimoNumeroPedido($row["CDFILLANC"], $row["CDFILDEST"]);
        $row["NEW_NRPEDEXOL"] = intval($row["NEW_NRPEDEXOL"]);
        $row["NEW_NRPEDEXOL"]++;
        $row["NEW_NRPEDEXOL"] = str_pad($row["NEW_NRPEDEXOL"], 10, 0, STR_PAD_LEFT);
        
        $param = array();
        $param["P_CDFILLANC"] = $row["CDFILLANC"];
        $param["P_CDFILDEST"] = $row["CDFILDEST"];
        $param["P_NRPEDEXOL"] = $row["NEW_NRPEDEXOL"];
        $param["P_DTPEDEXOL"] = $row["DTPEDEXOL"];
        $param["P_IDEMITEXOL"] = $row["IDTIPOEMITAF"];
        $param["P_IDTPFRETEXOL"] = isset($row["IDTIPOFRETAF"]) ? $row["IDTIPOFRETAF"] : null;
        $param["P_CDEMPRESA"] = $row["CDEMPRESA"];
        $param["P_CDTIPCOB"] = isset($row["CDTIPCOB"]) ? $row["CDTIPCOB"] : null;
        $param["P_IDTPJTREXOL"] = isset($row["IDTPIJURTRAN"]) ? $row["IDTPIJURTRAN"] : null;
        $param["P_NRINSJTREXOL"] = isset($row["NRINSJURTRAN"]) ? $row["NRINSJURTRAN"] : null;
        $param["P_CDTPCTPAGAR"] = isset($row["CDTPCTPAGAR"]) ? $row["CDTPCTPAGAR"] : null;
        $param["P_CDFILIOPELOG"] = $row["CDFILIOPELOG"];
        $param["P_IDTIPOEXOL"] = 'M';
        $param["P_IDPEDEXOLEXTR"] = $row["IDPEDEXOLEXTR"];
        
        $query = "INSERT INTO PEDIDOEXOL (CDFILDEST, CDFILLANC, NRPEDEXOL, DTPEDEXOL, IDEMITEXOL, IDTPFRETEXOL, CDEMPRESA, CDTIPCOB, IDTPJTREXOL, NRINSJTREXOL, CDTPCTPAGAR, CDFILIOPELOG, IDTIPOEXOL, IDPEDEXOLEXTR) 
				VALUES (:P_CDFILDEST, :P_CDFILLANC, :P_NRPEDEXOL, :P_DTPEDEXOL, :P_IDEMITEXOL, :P_IDTPFRETEXOL, :P_CDEMPRESA, :P_CDTIPCOB, :P_IDTPJTREXOL, :P_NRINSJTREXOL, :P_CDTPCTPAGAR, :P_CDFILIOPELOG, :P_IDTIPOEXOL, :P_IDPEDEXOLEXTR)";
                     
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            
            return $row["NEW_NRPEDEXOL"];
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function getFatorPreco() {
        $param = array();
        $param["P_SGFATPRECO"] = null;
        $param["P_TSGFATPRECO"] = 'T';
        
        $query = "SELECT CDFATPRECO, SGFATPRECO, NMFATPRECO
                      FROM FATORPRECO
                     WHERE ((SGFATPRECO = :P_SGFATPRECO) OR (:P_TSGFATPRECO = 'T'))
                     ORDER BY SGFATPRECO";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            $return = $std->fetchAll();
            if(isset($return[0]))
                return $return[0];
            return null;
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function procedureGeraPedOLExpedicao($row) {
        try {
            $procedure = new StoredProcedure($this->connection, "GERA_PED_OL_EXPEDICAO");
        
            $procedure->addParam(new Param("P_CDFILDEST", Param::PARAM_INPUT, $row["CDFILDEST"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFILLANC", Param::PARAM_INPUT, $row["CDFILLANC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_NRPEDEXOL", Param::PARAM_INPUT, $row["NRPEDEXOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFATPRECO", Param::PARAM_INPUT, $row["CDFATPRECO"], Param::PARAM_TYPE_STR));
            
            return $procedure->execute();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function procedureGeraNovoCodigo($tabela) {
        try {
            //return $this->newCode->getCodeByTableName($p_contador, $this->environment->getNrorgTrab(), 1, 10, false);
            return $this->newCode->getCodeByTableName($tabela, $this->environment->getNrOrg());
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function insereNovaLista($codigo, $row) {
        $param = array();
        $param["P_NRLISTA"] = $codigo;
        $param["P_CDGRUPFILI"] = isset($row["CDGRUPFILIBKP"]) ? $row["CDGRUPFILIBKP"] : null;
        
        $query = "INSERT INTO LISTA (NRLISTA, CDLISTA)
                  SELECT :P_NRLISTA, CDFILIAL
                    FROM FILIAL
                   WHERE :P_CDGRUPFILI IS NULL
                   UNION 
                  SELECT :P_NRLISTA, FILGRUPFILI.CDFILIAL
                    FROM FILGRUPFILI
                   WHERE :P_CDGRUPFILI IS NOT NULL
                     AND INSTR(:P_CDGRUPFILI, FILGRUPFILI.CDGRUPFILI) > 0
                     AND NOT EXISTS (SELECT 1
                                       FROM LISTA L
                                      WHERE L.NRLISTA = :P_NRLISTA
                                        AND L.CDLISTA = FILGRUPFILI.CDFILIAL)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqValidacoes($codigo, $row) {
        $param = array();
        $param["P_DTINICIAL"] = (isset($row["DTPROGENPEOL"]["start"]) && !empty($row["DTPROGENPEOL"]["start"])) ? $row["DTPROGENPEOL"]["start"] : null;
        $param["P_DTFINAL"] = (isset($row["DTPROGENPEOL"]["end"]) && !empty($row["DTPROGENPEOL"]["end"])) ? $row["DTPROGENPEOL"]["end"] : null;
        $param["P_PRODINI"] = isset($row["CDPRODINI"]) ? $row["CDPRODINI"] : null;
        $param["P_PRODFIN"] = isset($row["CDPRODFIN"]) ? $row["CDPRODFIN"] : null;
        $param["P_CDFILLANC"] = $row["CDFILLANC"];
        $param["P_CDREGIAO"] = isset($row["CDREGIAOBKP"]) ? $row["CDREGIAOBKP"] : null;
        $param["P_CDGRUPFILI"] = isset($row["CDGRUPFILIBKP"]) ? $row["CDGRUPFILIBKP"] : null;
        $param["P_CDOPERADOR"] = $this->environment->getCdOperador();
        $param["P_CDGRUPPROD"] = isset($row["CDGRUPPRODBKP"]) ? $row["CDGRUPPRODBKP"] : null;
        $param["P_CDSUBGRPROD"] = isset($row["CDSUBGRPRODBKP"]) ? $row["CDSUBGRPRODBKP"] : null;
        $param["P_LISTA_FILIAL"] = $codigo;
        $param["P_EXTRA"] = $row["EXTRA"];
        
        $query = "SELECT COD_ERROR,  CDFILSOLI,    SUBSTR(NMFILIAL, 0, 10) NMFILIAL,
                         CDFILLAN,   NRSOLICMP,    DTPROGENTR,
                         QTSOLIPROD, CDPRODUTO,    CDARVPROD,
                         SGUNIDADE,  NMPRODUTO,    NRSEQUCONSOL,
                         IDPEDIDOOL, IDSOLIAPROV,  IDITMCONS
                    FROM TABLE(VERIFICA_INCONS_ITEMPEDEXOL(:P_DTINICIAL,
                                                           :P_DTFINAL,
                                                           :P_PRODINI,
                                                           :P_PRODFIN,
                                                           :P_CDFILLANC,
                                                           :P_CDREGIAO,
                                                           :P_CDGRUPFILI,
                                                           :P_CDOPERADOR,
                                                           :P_CDGRUPPROD,
                                                           :P_CDSUBGRPROD,
                                                           :P_LISTA_FILIAL,
                                                           :P_EXTRA))
                   ORDER BY CDFILSOLI,  CDFILLAN,  NRSOLICMP, 
                            DTPROGENTR, CDARVPROD, COD_ERROR";
                            
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function deleteNovaLista($codigo) {
        $param = array();
        $param["P_NRLISTA"] = $codigo;
        
        $query = "DELETE FROM
                       LISTA
                    WHERE
                       NRLISTA = :P_NRLISTA";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function incluiPedidoOLProcedure($row) {
        try {
            $procedure = new StoredProcedure($this->connection, "INCLUI_PEDIDO_OL");
        
            $procedure->addParam(new Param("P_DTINICIAL", Param::PARAM_INPUT, (isset($row["DTPROGENPEOL"]["start"]) && !empty($row["DTPROGENPEOL"]["start"])) ? $row["DTPROGENPEOL"]["start"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTFINAL", Param::PARAM_INPUT, (isset($row["DTPROGENPEOL"]["end"]) && !empty($row["DTPROGENPEOL"]["end"])) ? $row["DTPROGENPEOL"]["end"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_PRODINI", Param::PARAM_INPUT, isset($row["CDPRODINI"]) ? $row["CDPRODINI"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_PRODFIN", Param::PARAM_INPUT, isset($row["CDPRODFIN"]) ? $row["CDPRODFIN"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFILLANC", Param::PARAM_INPUT, $row["CDFILLANC"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDREGIAO", Param::PARAM_INPUT, isset($row["CDREGIAOBKP"]) ? $row["CDREGIAOBKP"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDGRUPFILI", Param::PARAM_INPUT, isset($row["CDGRUPFILIBKP"]) ? $row["CDGRUPFILIBKP"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDOPERADOR", Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDGRUPPROD", Param::PARAM_INPUT, isset($row["CDGRUPPRODBKP"]) ? $row["CDGRUPPRODBKP"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDSUBGRPROD", Param::PARAM_INPUT, isset($row["CDSUBGRPRODBKP"]) ? $row["CDSUBGRPRODBKP"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_ROWSAFFECT", Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
            $procedure->addParam(new Param("P_CDFATPRECO", Param::PARAM_INPUT, isset($row["CDFATPRECO"]) ? $row["CDFATPRECO"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_EXTRA", Param::PARAM_INPUT, isset($row["EXTRA"]) ? $row["EXTRA"] : null, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDLOTENVITOL", Param::PARAM_INPUT, isset($row["CDLOTENVITOL"]) ? $row["CDLOTENVITOL"] : null, Param::PARAM_TYPE_STR));
            
            return $procedure->execute();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function excluiItemPedidoOLItemSoli($row) {
        $CDFILDEST = explode(" - ", $row["CDFILDEST"]);
        
        $param = array();
        $param["P_CDFILDEST"]       = $CDFILDEST[0];
        $param["P_DTPROGENTR"]      = $row["DTPROGENPEOL"];
        $param["P_CDFILIOPELOG"]    = $row["CDFILIOPELOG"];
        $param["P_CDPRODESTO"]      = isset($row["CDARVPROD"]) ? $row["CDARVPROD"] : $row["CDPRODUTO"];
        
        $query = "UPDATE ITEMSOLI
                       SET IDPEDIDOOL = 'N'
                    WHERE ITEMSOLI.CDFILSOLI = :P_CDFILDEST
                      AND ITEMSOLI.DTPROGENTR = :P_DTPROGENTR
                      AND ITEMSOLI.CDFILIOPELOG = :P_CDFILIOPELOG
                      AND ITEMSOLI.IDPEDIDOOL = 'S'
                      AND ITEMSOLI.NRSEQUCONSOL IS NOT NULL
                      AND ITEMSOLI.CDFILIOPELOG <> ITEMSOLI.CDFILSOLI
                      AND EXISTS (SELECT 1
                                  FROM PRODUTO
                                  WHERE PRODUTO.CDPRODUTO = ITEMSOLI.CDPRODUTO
                                  AND PRODUTO.CDPRODESTO = :P_CDPRODESTO)";
                      
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqBloqueiaPedidoOL($row) {
        try {
            $param = array();
            $param["P_CDPRODINI"]       = $row["P_CDPRODINI"];
            $param["P_CDPRODFIN"]       = $row["P_CDPRODFIN"];
            $param["P_CDFILIOL"]        = $row["P_CDFILIOL"];
            $param["P_TCDFILIOL"]       = $row["P_TCDFILIOL"];
            $param["P_CDFILIAL"]        = $row["P_CDFILIAL"];
            $param["P_TCDFILIAL"]       = $row["P_TCDFILIAL"];
            $param["P_SUBGRUPO"]        = $row["P_SUBGRUPO"];
            $param["P_CDGRUPPROD"]      = $row["P_CDGRUPPROD"];
            $param["P_DTENTRINI"]       = $row["P_DTENTRINI"];
            $param["P_DTENTRFIN"]       = $row["P_DTENTRFIN"];
            $param["P_TDTENTR"]         = $row["P_TDTENTR"];
            $param["P_DTEMBINI"]        = $row["P_DTEMBINI"];
            $param["P_DTEMBFIN"]        = $row["P_DTEMBFIN"];
            $param["P_TDTEMB"]          = $row["P_TDTEMB"];
            $param["P_TROTA"]           = $row["P_ROTA"] == 'T' ? null : $row["P_ROTA"];
            $param["P_ROTA"]            = $row["P_ROTA"];
            $param["P_CDOPERADOR"]      = 'T';
            $param["P_TPRODUTO"]        = $row["P_PRODUTO"] == 'T' ? 'T' : null;
            $param["P_IDPEDEXTRA"]      = $row["P_PEDEXOLEXTR"];
            $param["P_CDLOTENVPEDOL"]   = $row["P_CDLOTENVITOL"];
            
            $query = "SELECT
                            ITEMPEDEXOL.CDFILDEST,
                            ITEMPEDEXOL.CDFILLANC,
                            ITEMPEDEXOL.NRPEDEXOL,
                            ITEMPEDEXOL.CDPRODUTO,
                            ITEMPEDEXOL.CDMARCA,
                            ITEMPEDEXOL.DTPROGENPEOL
                        FROM
                            PRODUTO,
                            PEDIDOEXOL,
                            ITEMPEDEXOL
                        WHERE
                                ITEMPEDEXOL.CDFILDEST = PEDIDOEXOL.CDFILDEST
                            AND ITEMPEDEXOL.CDFILLANC = PEDIDOEXOL.CDFILLANC
                            AND ITEMPEDEXOL.NRPEDEXOL = PEDIDOEXOL.NRPEDEXOL
                            AND ITEMPEDEXOL.CDPRODUTO = PRODUTO.CDPRODUTO
                            AND NVL(ITEMPEDEXOL.IDBAIXENPEOL, '1') = '1'
                            AND NVL(ITEMPEDEXOL.IDBAIXAOLSAI, '1') = '1'
                            AND ((PEDIDOEXOL.IDPEDEXOLEXTR = :P_IDPEDEXTRA)
                                  OR (:P_IDPEDEXTRA = 'T'))
                        AND ((INSTR(:P_CDLOTENVPEDOL, ITEMPEDEXOL.CDLOTENVITOL) > 0) OR (NVL(ITEMPEDEXOL.CDLOTENVITOL, :P_CDLOTENVPEDOL) = 'T'))
                                AND NVL(ITEMPEDEXOL.IDITEMAJUSTE, 'N') = 'N'
                                AND ((:P_TROTA = 'T') OR
                                     EXISTS (SELECT 1
                                               FROM FILIALROTA
                                                WHERE FILIALROTA.CDFILIROTA = ITEMPEDEXOL.CDFILDEST
                                                AND INSTR(:P_ROTA, FILIALROTA.CDROTATRANS) > 0))
                                AND ((:P_CDOPERADOR = 'T') OR
                                     EXISTS (SELECT 1
                                             FROM FILIOPER
                                             WHERE FILIOPER.CDOPERADOR = :P_CDOPERADOR
                                               AND FILIOPER.CDFILIAL = ITEMPEDEXOL.CDFILDEST))
                                  AND ((PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN) OR (:P_TPRODUTO = 'T'))
                                  AND (INSTR(:P_CDFILIOL, PEDIDOEXOL.CDFILIOPELOG) <> 0 OR :P_TCDFILIOL = 'T')
                                  AND (INSTR(:P_CDFILIAL, ITEMPEDEXOL.CDFILDEST) <> 0 OR :P_TCDFILIAL = 'T')
                                AND ((:P_SUBGRUPO = 'N') OR
                                     (EXISTS(SELECT 1
                                             FROM PRSUBGRPROD
                                             WHERE PRSUBGRPROD.CDGRUPPROD  = :P_CDGRUPPROD
                                                 AND INSTR(:P_SUBGRUPO, PRSUBGRPROD.CDSUBGRPROD) > 0 AND PRSUBGRPROD.CDPRODUTO = ITEMPEDEXOL.CDPRODUTO)))
                        AND ((ITEMPEDEXOL.DTPROGENPEOL BETWEEN :P_DTENTRINI AND :P_DTENTRFIN)
                              OR (:P_TDTENTR = 'T'))
                            AND ((ITEMPEDEXOL.DTPROGEMBARQ BETWEEN :P_DTEMBINI AND :P_DTEMBFIN)
                                  OR (:P_TDTEMB = 'T'))";
            
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmBloqueiaItemPedOL($item) {
        $param = array();
        $param["P_CDOPERADOR"]      = $this->environment->getCdOperador();
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILLANC"]       = $item["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $item["NRPEDEXOL"];
        $param["P_CDPRODUTO"]       = $item["CDPRODUTO"];
        $param["P_CDMARCA"]         = $item["CDMARCA"];
        $param["P_DTPROGENPEOL"]    = $item["DTPROGENPEOL"];
        
        $query = "UPDATE ITEMPEDEXOL
                         SET IDITMPEDOLBLOQ = 'S',
                             DTBLOQITMPEDOL = SYSDATE,
                             CDOPERBLOQITM  = :P_CDOPERADOR
                      WHERE CDFILDEST     = :P_CDFILDEST
                        AND CDFILLANC     = :P_CDFILLANC
                        AND NRPEDEXOL     = :P_NRPEDEXOL
                        AND CDPRODUTO     = :P_CDPRODUTO
                        AND CDMARCA       = :P_CDMARCA
                        AND DTPROGENPEOL  = :P_DTPROGENPEOL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqItemPedido($row) {
        try {
            $param = array();
            $param["P_CDPRODINI"]       = $row["P_CDPRODINI"];
            $param["P_CDPRODFIN"]       = $row["P_CDPRODFIN"];
            $param["P_CDFILIOL"]        = $row["P_CDFILIOL"];
            $param["P_TCDFILIOL"]       = $row["P_TCDFILIOL"];
            $param["P_CDFILIAL"]        = $row["P_CDFILIAL"];
            $param["P_TCDFILIAL"]       = $row["P_TCDFILIAL"];
            $param["P_SUBGRUPO"]        = $row["P_SUBGRUPO"];
            $param["P_CDGRUPPROD"]      = $row["P_CDGRUPPROD"];
            $param["P_DTENTRINI"]       = $row["P_DTENTRINI"];
            $param["P_DTENTRFIN"]       = $row["P_DTENTRFIN"];
            $param["P_TDTENTR"]         = $row["P_TDTENTR"];
            $param["P_DTEMBINI"]        = $row["P_DTEMBINI"];
            $param["P_DTEMBFIN"]        = $row["P_DTEMBFIN"];
            $param["P_TDTEMB"]          = $row["P_TDTEMB"];
            $param["P_TROTA"]           = $row["P_ROTA"] == 'T' ? null : $row["P_ROTA"];
            $param["P_ROTA"]            = $row["P_ROTA"];
            $param["P_CDOPERADOR"]      = 'T';
            $param["P_TPRODUTO"]        = $row["P_PRODUTO"] == 'T' ? 'T' : null;
            $param["P_IDPEDEXTRA"]      = $row["P_PEDEXOLEXTR"];
            $param["P_CDLOTENVPEDOL"]   = $row["P_CDLOTENVITOL"];
            
            $query = "SELECT
                    	ITEMPEDEXOL.CDFILDEST,
                    	ITEMPEDEXOL.CDFILLANC,
                    	ITEMPEDEXOL.NRPEDEXOL,
                    	ITEMPEDEXOL.CDPRODUTO,
                    	PRODUTO.NMPRODUTO,
                    	ITEMPEDEXOL.CDMARCA,
                    	ITEMPEDEXOL.DTPROGENPEOL,
                    	ITEMPEDEXOL.QTPROGENPEOL,
                    	PEDIDOEXOL.CDFILIOPELOG,
                    	ITEMPEDEXOL.DTPROGEMBARQ,
                    	FILIAL.NMFILIAL,
                    	FILDEST.NMFILIAL                            NMFILDEST,
                    	PRODUTO.SGUNIDADE,
                    	PRODUTO.CDARVPROD,
                    	NVL(ITEMPEDEXOL.IDBAIXAOLSAI, '0')          IDBAIXAOLSAI,
                    	NVL(ITEMPEDEXOL.IDENVITMOL, 'N')            IDENVITMOL,
                    	NVL(ITEMPEDEXOL.IDITMPEDOLBLOQ, 'N')        IDITMPEDOLBLOQ
                      FROM
                    	PRODUTO,
                    	PEDIDOEXOL,
                    	ITEMPEDEXOL,
                    	FILIAL,
                    	FILIAL FILDEST
                     WHERE
                    	    ITEMPEDEXOL.CDFILDEST = PEDIDOEXOL.CDFILDEST
                    		AND ITEMPEDEXOL.CDFILLANC                     = PEDIDOEXOL.CDFILLANC
                    		AND ITEMPEDEXOL.NRPEDEXOL                     = PEDIDOEXOL.NRPEDEXOL
                    		AND ITEMPEDEXOL.CDPRODUTO                     = PRODUTO.CDPRODUTO
                    		AND NVL(ITEMPEDEXOL.IDBAIXENPEOL, '1')        = '1'
                    		AND NVL(ITEMPEDEXOL.IDBAIXAOLSAI, '1')        = '1'
                    		AND PEDIDOEXOL.CDFILIOPELOG                   = FILIAL.CDFILIAL
                    		AND ITEMPEDEXOL.CDFILDEST                     = FILDEST.CDFILIAL
                    		AND ((PEDIDOEXOL.IDPEDEXOLEXTR = :P_IDPEDEXTRA)
                    		OR (:P_IDPEDEXTRA = 'T'))
                    		AND ((INSTR(:P_CDLOTENVPEDOL, ITEMPEDEXOL.CDLOTENVITOL) > 0) OR (NVL(ITEMPEDEXOL.CDLOTENVITOL, :P_CDLOTENVPEDOL) = 'T'))
                            AND NVL(ITEMPEDEXOL.IDITEMAJUSTE, 'N') = 'N'
                            AND ((:P_TROTA = 'T') OR
                                 EXISTS (SELECT 1
                                           FROM FILIALROTA
                                            WHERE FILIALROTA.CDFILIROTA = ITEMPEDEXOL.CDFILDEST
                                              AND INSTR(:P_ROTA, FILIALROTA.CDROTATRANS) > 0))
                            AND ((:P_CDOPERADOR = 'T') OR
                                 EXISTS (SELECT 1
                                         FROM FILIOPER
                                         WHERE FILIOPER.CDOPERADOR = :P_CDOPERADOR
                                           AND FILIOPER.CDFILIAL = ITEMPEDEXOL.CDFILDEST))
                              AND ((PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN) OR (:P_TPRODUTO = 'T'))
                              AND (INSTR(:P_CDFILIOL, PEDIDOEXOL.CDFILIOPELOG) <> 0 OR :P_TCDFILIOL = 'T')
                              AND (INSTR(:P_CDFILIAL, ITEMPEDEXOL.CDFILDEST) <> 0 OR :P_TCDFILIAL = 'T')
                            AND ((:P_SUBGRUPO = 'N') OR
                                 (EXISTS(SELECT 1
                                         FROM PRSUBGRPROD
                                         WHERE PRSUBGRPROD.CDGRUPPROD  = :P_CDGRUPPROD
                                             AND INSTR(:P_SUBGRUPO, PRSUBGRPROD.CDSUBGRPROD) > 0 AND PRSUBGRPROD.CDPRODUTO = ITEMPEDEXOL.CDPRODUTO)))
                    AND ((ITEMPEDEXOL.DTPROGENPEOL BETWEEN :P_DTENTRINI AND :P_DTENTRFIN)
                        OR (:P_TDTENTR = 'T'))
                       AND ((ITEMPEDEXOL.DTPROGEMBARQ BETWEEN :P_DTEMBINI AND :P_DTEMBFIN)
                        OR (:P_TDTEMB = 'T'))
                            ORDER BY PEDIDOEXOL.CDFILIOPELOG,
                                                          ITEMPEDEXOL.DTPROGEMBARQ,
                                                          PRODUTO.CDARVPROD,
                                                          ITEMPEDEXOL.CDMARCA,
                                                          ITEMPEDEXOL.CDFILLANC,
                                                          ITEMPEDEXOL.CDFILDEST,
                                                          ITEMPEDEXOL.NRPEDEXOL";
            
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqParasupri($central) {
        $param = array();
        $param["P_CDFILCENCMP"] = $central;
        
        $query = "SELECT NVL(IDUTRESPEDEXOL, 'N') IDUTRESPEDEXOL, IDUTCTRLFISOL,
                	   NVL(IDUTDADOPERENVEMAIL, 'LOGIN_FILIAL_REMETENTE_OPER') IDUTDADOPERENVEMAIL 
                	FROM PARASUPRI
                	WHERE CDFILIAL = :P_CDFILCENCMP";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqVdRelacprodol($item) {
        $param = array();
        $param["P_CDPRODUTO"]   = $item["CDPRODUTO"];
        $param["P_CDMARCA"]     = $item["CDMARCA"];
        
        $query = "SELECT CDPRODARVOL, CDMARCA
                    FROM RELACPRODOL
                   WHERE CDPRARVORIG = :P_CDPRODUTO
                     AND CDMARCA     = :P_CDMARCA";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqParamIntegra($item) {
        $param = array();
        $param["P_CDFILIAL"] = $item["CDFILIOPELOG"];
        
        $query = "SELECT CDFILIAL
                    FROM PARAMFILIAL
                   WHERE CDFILIAL = :P_CDFILIAL
                     AND CDFILWMSMHA  IS NOT NULL
                     AND CDOPERWMSMHA IS NOT NULL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdAtCompra($item) {
        $param = array();
        $param["P_CDPRODUTO"] = $item["CDPRODUTO"];
        
        $query = "SELECT 1
                    FROM PRODUTO
                   WHERE IDPRCMPATIVO = 'S'
                     AND CDPRODUTO    = :P_CDPRODUTO";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdConv($item) {
        $param = array();
        $param["P_CDPRODUTO"] = $item["CDPRODARVOL"];
        
        $query = "SELECT 1
                    FROM CNVPRODUNEST
                   WHERE (CDPRODUTO = :P_CDPRODUTO) OR (CDPRUNEST = :P_CDPRODUTO)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdNPeso($item) {
        $param = array();
        $param["P_CDPRODUTO"] = $item["CDPRODARVOL"];
        
        $query = "SELECT 1 FROM PRODUTO
                  WHERE CDPRODUTO = :P_CDPRODUTO
                    AND ((NVL(VRPESOBRUTPR, 0) > 0) OR (NVL(VRPESOLIQUPR, 0) > 0))";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqFiliAgentTerceiro($item) {
        $param = array();
        $param["P_CDFILIOPELOG"] = $item["CDFILIOPELOG"];
        
        $query = "SELECT NVL(MAX(CDFILIAL), :P_CDFILIOPELOG) CDFILIAL
                   FROM FILIAL
                  WHERE CDFILIOLAGETERC = :P_CDFILIOPELOG";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqUltimoMovEst($item, $filial) {
        $param = array();
        $param["P_CDPRODUTO"]   = $item["CDPRODARVOL"];
        $param["P_CDFILIAL"]    = $filial["CDFILIAL"];
        
        $query = "SELECT NVL(MAX(DTPOSESTCON), SYSDATE + 1) AS DTPOSESTCON
                    FROM POSESTCONS
                   WHERE CDFILIAL  = :P_CDFILIAL
                     AND CDPRODUTO = :P_CDPRODUTO";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdEstoque($item, $filial, $data) {
        $param = array();
        $param["P_CDPRODUTO"]       = $item["CDPRODARVOL"];
        $param["P_CDFILIOPELOG"]    = $filial["CDFILIAL"];
        $param["P_DTENTRFIN"]       = $data["DTPOSESTCON"];
        
        $query = "SELECT NVL(CALC_ESTOQ_CONS(:P_CDFILIOPELOG, :P_CDPRODUTO, TRUNC(:P_DTENTRFIN) + 1), 0) ESTOQ_DISPO FROM DUAL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdReserva($item, $filial) {
        $param = array();
        $param["P_CDFILIOPELOG"]    = $filial["CDFILIAL"];
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILLANC"]       = $item["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $item["NRPEDEXOL"];
        $param["P_CDMARCA"]         = $item["CDMARCA"];
        $param["P_CDPRODUTO"]       = $item["CDPRODUTO"];
        $param["P_DTPROGENPEOL"]    = $item["DTPROGENPEOL"];
        
        $query = "SELECT SUM(RE.QTRESERVESTOL) ESTOQ_RESERVADO
                    FROM RESERVAPEDEXOL RE
                   WHERE (RE.CDFILDEST   <> :P_CDFILDEST
                      OR RE.CDFILLANC    <> :P_CDFILLANC
                      OR RE.NRPEDEXOL    <> :P_NRPEDEXOL
                      OR RE.DTPROGENPEOL <> :P_DTPROGENPEOL)
                     AND RE.CDPRODUTO    =  :P_CDPRODUTO
                     AND RE.CDMARCA      =  :P_CDMARCA
                     AND RE.CDFILRESOL   =  :P_CDFILIOPELOG";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqReserva($item) {
        $param = array();
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILLANC"]       = $item["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $item["NRPEDEXOL"];
        $param["P_CDMARCA"]         = $item["CDMARCA"];
        $param["P_CDPRODUTO"]       = $item["CDPRODUTO"];
        $param["P_DTPROGENPEOL"]    = $item["DTPROGENPEOL"];
        
        $query = "SELECT RE.QTRESERVESTOL
                    FROM RESERVAPEDEXOL RE
                   WHERE RE.CDFILDEST    =  :P_CDFILDEST
                     AND RE.CDFILLANC    =  :P_CDFILLANC
                     AND RE.NRPEDEXOL    =  :P_NRPEDEXOL
                     AND RE.DTPROGENPEOL =  :P_DTPROGENPEOL
                     AND RE.CDPRODUTO    =  :P_CDPRODUTO
                     AND RE.CDMARCA      =  :P_CDMARCA";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqParametro() {
        $param = array();
        $param["P_CDFILIAL"] = $this->environment->getCdFilial();
        
        $query = "SELECT P.IDUTILCONSSOLIOL
                    FROM PARASUPRI P, FILIAL F
                   WHERE F.CDFILIAL = :P_CDFILIAL
                     AND P.CDFILIAL = NVL(F.CDFILCENTCMP, :P_CDFILIAL)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function acmInsReserva($item) {
        $param = array();
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILLANC"]       = $item["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $item["NRPEDEXOL"];
        $param["P_CDPRODUTO"]       = $item["CDPRODUTO"];
        $param["P_CDMARCA"]         = $item["CDMARCA"];
        $param["P_DTPROGENPEOL"]    = $item["DTPROGENPEOL"];
        $param["P_QTPROGENPEOL"]    = $item["QTPROGENPEOL"];
        $param["P_CDFILIOPELOG"]    = $filial["CDFILIAL"];
        $param["P_DTPROGEMBARQ"]    = $item["DTPROGEMBARQ"];
        
        $query = "INSERT INTO RESERVAPEDEXOL (CDFILDEST, CDFILLANC, NRPEDEXOL, CDMARCA, CDPRODUTO, DTPROGENPEOL, QTRESERVESTOL, CDFILRESOL, DTRESEMBARQ)
                    SELECT :P_CDFILDEST, :P_CDFILLANC, :P_NRPEDEXOL, :P_CDMARCA, :P_CDPRODUTO, :P_DTPROGENPEOL, :P_QTPROGENPEOL, :P_CDFILIOPELOG, :P_DTPROGEMBARQ
                    FROM DUAL
                    WHERE EXISTS (SELECT 1
                                           FROM ITEMPEDEXOL
                                          WHERE ITEMPEDEXOL.CDFILDEST    = :P_CDFILDEST
                                            AND ITEMPEDEXOL.CDFILLANC    = :P_CDFILLANC
                                            AND ITEMPEDEXOL.NRPEDEXOL    = :P_NRPEDEXOL
                                            AND ITEMPEDEXOL.CDMARCA      = :P_CDMARCA
                                            AND ITEMPEDEXOL.CDPRODUTO    = :P_CDPRODUTO
                                            AND ITEMPEDEXOL.DTPROGENPEOL = :P_DTPROGENPEOL)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqParam($item) {
        $param = array();
        $param["P_CDFILIAL"] = $item["CDFILIOPELOG"];
        
        $query = "SELECT NVL(CDTIPCUSTEST, '3') CDTIPCUSTEST
                    FROM PARAMFILIAL
                   WHERE CDFILIAL = :P_CDFILIAL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqCusto($item, $vdItem, $custo) {
        $param = array();
        $param["P_CDFILIOPELOG"]    = $item["CDFILIOPELOG"];
        $param["DTPROGENPEOL"]      = $item["DTPROGENPEOL"];
        $param["P_CDPRODUTO"]       = $vdItem["CDPRODARVOL"];
        $param["P_CDTIPCUSTEST"]    = $custo["CDTIPCUSTEST"];
        
        $query = "SELECT CUSTO_PRODUTO(:P_CDFILIOPELOG, :P_CDPRODUTO, :P_CDTIPCUSTEST, :DTPROGENPEOL, SYSDATE) VRCUSTO FROM DUAL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqPrClassFisc($vdItem) {
        $param = array();
        $param["P_CDPRODUTO"] = $vdItem["CDPRODARVOL"];
        
        $query = "SELECT CDCLASFISC FROM PRODUTO WHERE CDPRODUTO = :P_CDPRODUTO AND CDCLASFISC IS NOT NULL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqAgrupamento($item) {
        $param = array();
        $param["P_CDFILDEST"] = $item["CDFILDEST"];
        $param["P_CDPRODUTO"] = $item["CDPRODUTO"];
        
        $query = "SELECT 1
                    FROM FILIAL, PARASUPRI, PRSUBGRPROD
                   WHERE FILIAL.CDFILIAL         = :P_CDFILDEST
                     AND PARASUPRI.CDFILIAL      = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                     AND PARASUPRI.CDGRUPPROD    = PRSUBGRPROD.CDGRUPPROD
                     AND PRSUBGRPROD.CDPRODUTO   = :P_CDPRODUTO
                     AND PRSUBGRPROD.CDFILISUBGR = 'P'
                     AND PARASUPRI.CDSUBGRARVOR  <> PRSUBGRPROD.CDSUBGRPROD";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqTipoOperOL($item) {
        $param = array();
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILIOPELOG"]    = $item["CDFILIOPELOG"];
        
        $query = "SELECT CDFILIOPELOG, CDFILDEST, CDTIPOOPERSAID
                    FROM TIPOOPEROPL
                   WHERE CDFILDEST      = :P_CDFILDEST
                     AND CDFILIOPELOG   = :P_CDFILIOPELOG
                     AND IDTIPOOPERSAID = 'S'
                     AND CDTIPOOPERSAID IS NOT NULL";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adsValida_Fiscal_OL($item, $tipoOper, $stNrErFisOL) {
        try {
            $procedure = new StoredProcedure($this->connection, "VALIDA_PARAM_FISCAL_OL");
        
            $procedure->addParam(new Param("P_NRERFISCALOL", Param::PARAM_INPUT, $stNrErFisOL, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFILIOPELOG", Param::PARAM_INPUT, $item["CDFILIOPELOG"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDFILDEST", Param::PARAM_INPUT, $item["CDFILDEST"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDTIPOOPER", Param::PARAM_INPUT, $tipoOper["CDTIPOOPERSAID"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_IDENTRSAIDOP", Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDARVPROD", Param::PARAM_INPUT, $item["CDARVPROD"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_CDMARCA", Param::PARAM_INPUT, $item["CDMARCA"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTERFISCALOL", Param::PARAM_INPUT, $item["DTPROGENPEOL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param("P_DTEMBRFISCOL", Param::PARAM_INPUT, $item["DTPROGEMBARQ"], Param::PARAM_TYPE_STR));
            
            return $procedure->execute();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqErroFiscalOL($item) {
        $param = array();
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILIOPELOG"]    = $item["CDFILIOPELOG"];
        $param["P_CDPRODUTO"]       = $item["CDARVPROD"];
        
        $query = "SELECT MAX(CDFILIOPELOG) CDFILIOPELOG
                    FROM ERROFISCALOL
                   WHERE CDFILIOPELOG = :P_CDFILIOPELOG
                     AND CDFILDEST    = :P_CDFILDEST
                     AND CDARVPRODER  = :P_CDPRODUTO";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
    
    public function adqProdAgrup($item, $row) {
        $param = array();
        $param["P_CDFILDEST"]       = $item["CDFILDEST"];
        $param["P_CDFILLANC"]       = $item["CDFILLANC"];
        $param["P_NRPEDEXOL"]       = $item["NRPEDEXOL"];
        $param["P_DTPROGENPEOL"]    = $item["DTPROGENPEOL"];
        $param["P_CDSUBGRPROD"]     = $row["P_SUBGRUPO"];
        
        $query = "SELECT I.CDPRODUTO, I.CDMARCA, I.DTPROGENPEOL, I.CDFILDEST, I.CDFILLANC, I.NRPEDEXOL,
                       I.DTPROGEMBARQ, I.QTPROGENPEOL, P.CDFILIOPELOG, PR.CDARVPROD, PR.SGUNIDADE, PR.NMPRODUTO
                  FROM PEDIDOEXOL P, ITEMPEDEXOL I, PRODUTO PR
                       WHERE P.CDFILDEST    = I.CDFILDEST
                         AND P.CDFILLANC    = I.CDFILLANC
                         AND P.NRPEDEXOL    = I.NRPEDEXOL
                         AND I.DTPROGENPEOL = :P_DTPROGENPEOL
                         AND P.CDFILDEST    = :P_CDFILDEST
                         AND P.CDFILLANC    = :P_CDFILLANC
                         AND P.NRPEDEXOL    = :P_NRPEDEXOL
                         AND I.CDPRODUTO    = PR.CDPRODUTO
                         AND NVL(I.IDITEMAJUSTE, 'N') = 'N'
                         AND NOT EXISTS (SELECT 1
                                           FROM FILIAL, PARASUPRI, PRSUBGRPROD
                                          WHERE FILIAL.CDFILIAL       = I.CDFILDEST
                                          AND PARASUPRI.CDFILIAL      = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)
                                          AND PARASUPRI.CDGRUPPROD    = PRSUBGRPROD.CDGRUPPROD
                                          AND PRSUBGRPROD.CDSUBGRPROD = SUBSTR(:P_CDSUBGRPROD, 2, 10)
                                          AND PRSUBGRPROD.CDFILISUBGR = 'P'
                                          AND PARASUPRI.CDSUBGRARVOR  <> PRSUBGRPROD.CDSUBGRPROD)";
        
        try {
            $std = $this->connection->prepare($query);
            $std->execute($param);
            return $std->fetchAll();
        } catch(Exception $e) {
            throw $e;
        }
    }
}